### A Pluto.jl notebook ###
# v0.20.4

using Markdown
using InteractiveUtils

# This Pluto notebook uses @bind for interactivity. When running this notebook outside of Pluto, the following 'mock version' of @bind gives bound variables a default value (instead of an error).
macro bind(def, element)
    #! format: off
    quote
        local iv = try Base.loaded_modules[Base.PkgId(Base.UUID("6e696c72-6542-2067-7265-42206c756150"), "AbstractPlutoDingetjes")].Bonds.initial_value catch; b -> missing; end
        local el = $(esc(element))
        global $(esc(def)) = Core.applicable(Base.get, el) ? Base.get(el) : iv(el)
        el
    end
    #! format: on
end

# ╔═╡ a145e2fe-a903-11eb-160f-df7ea83fa3e6
using PlutoUI, PlutoPlotly, CSV, DataFrames, Dates, StatsBase, SparseArrays

# ╔═╡ d092fde8-d4cd-40a3-a55d-ab06d322b9fa
html"""
<picture>
  <source srcset="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/negative/rgb_ibs_en_horizontal_negative.png" media="(prefers-color-scheme: dark)" width="240px!important" style="float:right;">
  <img src="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/main/rgb_ibs_en_horizontal_main.png" width="240px" style="float:right;">
</picture>
"""

# ╔═╡ 943edb36-acc8-4b2a-94ab-c544f9eb279b
md"""
# Week 3 - Measuring Macroeconomic Activity

## Exercises

**Macroeconomics, ISCTE-IUL**
"""

# ╔═╡ 1245aef9-18ce-42f5-af82-c2e6cce70798
md"""
**Vivaldo Mendes, Ricardo Gouveia-Mendes, Luís Clemente-Casinhas**

**February 2025**
"""

# ╔═╡ aa744544-1a69-45b4-aee6-7b9668d7f3cd
md"""
---
"""

# ╔═╡ 4618a663-f77e-4ccc-9b00-e84dbd5beb34
md"""
### Packages used in this notebook
"""

# ╔═╡ 5930d220-05cc-44ee-946c-55b613b235f8
md"""
---
"""

# ╔═╡ dcd13560-0f73-4aaa-b59a-b7961d8cdc63
md"""
## Exercise 1. What counts as GDP? I
"""

# ╔═╡ df9417d2-ebe1-498c-b299-f2b8fdee1084
md"""
*From the textbook "Macroeconomics" 4th Edition, by Charles I. Jones, Chapter 2*

By how much does GDP rise in each of the following
scenarios? Explain.

(a) You spend \$5000 on college tuition this semester.

(b) You buy a used car from a friend for \$2500.

(c) The government spends \$100 million to build a dam.

(d) Foreign graduate students work as teaching assistants at the local university and earn \$5,000 each.
"""

# ╔═╡ cec1d956-027e-445e-9b9e-db2aacfe71b1
md"""
!!! answers

	(a) ΔGDP = \$5000. You spent $5000 on a final service this semester.

	(b) ΔGDP = \$0. Transactions of used goods are not part of the current GDP.

	(c) ΔGDP = \$100 million. The government spent that amount on a final good.

	(d) ΔGDP = \$5000 per student. This university spent $5000 on a final service provided by each student.

"""

# ╔═╡ 0ac99d4a-f5eb-41fd-bed2-4d7ec42ac19c
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 9e05bc6e-a423-41c2-a236-41432276af88
md"""
## Exercise 2. What counts as GDP? II
"""

# ╔═╡ 72e235dd-e402-483b-9567-28fb05c364f8
md"""

*From the textbook "Macroeconomics", 4th Edition, by Charles I. Jones, Chapter 2*

By how much does GDP rise in each of the following
scenarios? Explain.

(a) A computer company buys parts from a local distributor for \$1 million, assembles the parts, and sells the resulting computers for \$2 million.

(b) A real estate agent sells a house for \$200,000 that the previous owners had bought 10 years earlier for $100,000. The agent earns a commission of \$6000.

(c) During a recession, the government raises unemployment benefits by \$100 million.

(d) A new U.S. airline purchases and imports \$50 million worth of airplanes from the European company Airbus.

(e) A new European airline purchases \$50 million worth of airplanes from the American company Boeing.

(f) A store buys \$100,000 of chocolate from Belgium and sells it to consumers in the United States for \$125,000.
"""

# ╔═╡ 96e1035c-a937-4385-b5bd-18ee56204651
md"""
!!! answers

	(a) ΔGDP = \$2 million. 

	- From the expenditure perspective: \$2 million as the sales of computers as a final good. 

	- From the production perspective: \$1 million value added by the producers of components, plus \$1 million value added by the firm that assembled the computers. 

	(b) ΔGDP = \$6000. The only final service or good produced during the current period is the real estate agent's service.

	(c) ΔGDP = \$0. Income transfers can not count to GDP; otherwise, GDP could be artificially increased by a constant movement of income transfers. 

	(d) ΔGDP = $-$\$50 million in GDP due to of Imports $+$ \$50 million in GDP due to Invesment = \$0.
		
	(e) ΔGDP = \$50 million. Exports increase by this amount.

	(f) ΔGDP = \$25000. 

	- By the expenditure approach, GDP declines by $-$\$100000 (due to Imports) but increases by $+$ \$125000 due to the final Consumption of goods. 

	- By the production approach, the value added by the store is \$25000.

"""

# ╔═╡ 7f063405-715a-498e-8842-ae3f06a2b058
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ adb106a5-23f6-4a1c-b4f0-2f7d26b51709
md"""
## Exercise 3. Approaches to estimate GDP
"""

# ╔═╡ 7b68feda-27a4-4ef5-8faf-698be306e3f0
md"""

Consider a simple economy with only two goods: wheat and bread. It is known that this economy produces \$500 of wheat and \$1000 of bread, and bakers buy all the wheat for producing bread. 

(a) What approaches can be used to calculate the level of GDP in this economy?

(b) What is the total amount of sales of final goods in this economy?

(c) How much of intermediate goods are traded?

(d) What is the total sales volume in this economy?

(e) What is the value-added produced by the bakers and by the farmers?

"""

# ╔═╡ a3668c3c-b417-4fc0-8708-5b4bdd937969
md"""
!!! answers

	(a) Production and expenditure approaches. 

	(b) \$1000. 

	(c) \$500. 

	(d) \$500 + \$1000 million = \$1500.
		
	(e) \$500 each. 
"""

# ╔═╡ 4e61281c-a3f0-415b-934d-3dacc3750357
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 781d8783-f530-4859-b531-7e1495c53cc4
md"""
## Exercise 4. Pandora vs Utopia
"""

# ╔═╡ 8b753d01-766e-437b-8c45-80a8444564ca
md"""

*From the textbook* 

The inhabitants of Pandora value their natural environment (e.g., forests, springs, breathable air, etc.) twice as much as the inhabitants of Utopia. Suppose that the **value added** for all goods and services *increases by the same amount in both countries*, but has a negative effect on the environment (e.g., pollution).

(a) According to the production approach to the measurement of GDP, is this good or bad?

(b) Are both countries necessarily better off? Which country benefits more for sure?

(c) The inhabitants of Utopia are very concerned about income distribution, which is not that important for the inhabitants of Pandora. If the increase in value added results in further wealth concentration, how
will this affect your answers to part (b)?
"""

# ╔═╡ 1552756d-2d14-47f8-a537-acd59a4031d4
md"""
!!! tip "Answer (a)"

	If a value-added increases by the same amount in both countries, the level of real GDP in both countries will be the same. Therefore, if we measure the value of what is economically produced by "value added", then pollution will have no impact on the measure of GDP in both countries.

"""

# ╔═╡ 52b2eb49-bcec-4901-bb77-39809063af4b
md"""
!!! tip "Answer (b)"

	If pollution is the same in both countries, they will experience the same level of utility from consumption (or the bad aspects of pollution). The point is that Pandora will have a much lower utility (or satisfaction) level than Utopia because it values its environment twice as much. Therefore, if we consider the adverse effects of pollution, the people in Pandora will feel worse off than those in Utopia. 

"""

# ╔═╡ d4e6bd4d-b4c0-4817-b53c-4539528d9b00
md"""
!!! tip "Answer (c)"

	The situation will be the opposite of that in (b). Suppose the measurement of GDP does not consider problems related to income distribution; two economies with the same GDP per capita may display different levels of utility (satisfaction). Therefore, if we consider the adverse effects of income distribution, the people in Utopia will feel worse off than those in Pandora.

"""

# ╔═╡ ac61e4b5-0082-47a6-a868-812208e35ee6
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 97a85466-47ff-4869-ba43-a36dea571284
md"""
## Exercise 5. GDP and its shares
"""

# ╔═╡ 839c6baf-dad8-4a97-97ea-d5da508c4263
md"""
According to the Expenditure approach to measure the level of GDP, what do you think about the following sentence:

*"One of the themes in my new study, “Why the Federal Government Fails,” is that the federal government has grown too large to manage with any reasonable level of efficiency and competence. Even if politicians worked diligently to advance the general interest, and even if federal bureaucracies focused on delivering quality services, the vast size of the government would still generate failure after failure."*

by Chris Edwards, "Federal Government: Too Big to Manage", July 2015, 
[The Cato Institute](https://www.cato.org/blog/federal-government-too-big-manage)

We provide some evidence in the following plots obtained with data from FRED.
"""

# ╔═╡ a66f7887-82b2-4826-a8a2-5a95fb070b84
GDP_shares = CSV.read("GDP_shares.csv", DataFrame)

# ╔═╡ d7bce0a6-1a1b-4626-ae34-e546154dcbec
begin
	fig5_1 = plot(GDP_shares.Quarters, GDP_shares.CONS, 
		Layout(height = 450,
			title_text = "Personal consumption expenditures as a % of GDP (US)", 
			title_x = 0.5, 
			hovermode = "x", 
			yaxis_title = "Percentage points", 
			xaxis_title = "Quarterly observations"
		)
	)
end


# ╔═╡ a7e5e83a-df8f-4aad-91f9-687a70de5b60
begin
	fig5_2 = plot(GDP_shares.Quarters, GDP_shares.GOV, 
		marker_color = "#0e6142", 
		Layout(height = 450,
			title_text = "Government consumption expenditures and gross investment as a % of GDP (US)", 
			title_x = 0.5, 
			hovermode = "x", 
			yaxis_title = "Percentage points", 
			xaxis_title = "Quarterly observations"
		)
	)
end

# ╔═╡ d937a483-f279-4ceb-93c9-3d72907a8e76
md"""
	!!! answer "Suggested answer"

		Chris Edwards does not indicate what aggregate he uses to derive his conclusions about the increasing size of the government in the US economy. From a macroeconomic point of view, the most natural candidate is the GDP share of government spending on goods & services and public investment spending because they represent the proportion of GDP that is "consumed"  by the state.

		Using the plots above, we can observe that the conclusion by Edwards looks suspicious. In the US, the share of public spending as a percentage of GDP has historically decreased, not increased. Furthermore, an increase in private spending seems to have compensated for the decline in public spending.

		When making assertions like those by Edwards, we must prove our points with data and facts. Otherwise, we will turn economics into a branch of politics.

	"""

# ╔═╡ 4699b6b5-8c87-4bd0-ac04-5b512b89ba5e
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 4d3108a1-32c0-4400-bc7c-375e0fbcf2ab
md"""
## Exercise 6. GDP, GNP, and Welfare
"""

# ╔═╡ 32cd7b4b-434a-4f35-b732-b6f85a8dc62d
md"""
Consider a country whose Gross Domestic Product (GDP) was 2 trillion dollars in 2023. From previous investments abroad, the residents of this country received an income of 0.5 trillion dollars in the same year from abroad. On the other hand, investments made in this country by residents in foreign countries led to an outflow of income of 0.1 trillion dollars. 
"""

# ╔═╡ 78e76c43-6061-48a2-a0d6-c7cc3212e846
md"""
👉 **a)** Calculate this country's aggregate defined as "Net Factor Income" in 2023.
"""

# ╔═╡ c089ebb2-9d11-4dfc-814b-f2bf0b08351a
md"""
!!! tip "Answer (a)"
		
	Net Factor Income = $0.5 - 0.1 = 0.4$ trillion dollars

"""

# ╔═╡ af36a976-2a19-4e0b-9dd9-958434b0aef3
md"""
👉 **b)** Calculate the level of Gross National Product (GNP) for this country in 2023.
"""

# ╔═╡ 3f10feb7-8948-4325-9a6d-b69660d22d65
md"""
!!! tip "Answer (b)"
		
	GNP = GDP + NFI = $2 + 0.4 = 2.4$ trillion dollars in 2023.

"""

# ╔═╡ dde1db83-f7dc-4d0f-936d-3adf32dc95b7
md"""
👉 **c)** For this country, in your opinion, what is the best aggregate to consider when we want to compare its welfare with other countries? GDP or GNP?
"""

# ╔═╡ 9b633f1a-e96e-414f-81ea-dfc99c1eb465
md"""
!!! tip "Answer (c)"
		
	A good indicator of the level of welfare citizens enjoy in a country is GDP or GNP per capita. In the current case, GDP per capita is only 83.33% of GNP per capita. Therefore, GNP per capita looks like a better indicator if the residents are relatively stable in this country. However, if we expect that some high-income earners from investments abroad will move to a foreign country, then the best welfare measure would be GDP per capita because it tells what the economy can produce internally. 

"""

# ╔═╡ 9b01b341-ef94-45c9-a921-a3efd4c0d50a
md"""
👉 **d)** Consider the case of the Republic of Ireland. Official statistics for 2023 tell us that GNP is only 76% of its GDP level. In Ireland, this disparity between GDP and GNP is caused by the fact that many international corporations have legal headquarters there. What aggregate do you consider best to describe the welfare in this country?
"""

# ╔═╡ 6417e9cc-d4aa-4e21-9188-19f8ea23ad74
md"""
!!! tip "Answer (d)"
	
	Those corporations have headquarters in Ireland because they pay much lower capital taxes than in their original countries. If the tax structure changes radically in various countries, these corporations may move away from Ireland, and the Irish GDP will suffer a sharp reduction. Therefore, a prudent indicator of welfare would be GNP rather than GDP in the case of Ireland.

"""

# ╔═╡ cbefa2ab-40bb-4c71-a147-00cd779259c5
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 640dd806-873f-4868-b5a8-0db0992e113d
md"""
## Exercise 7. Income vs Product

"""

# ╔═╡ 5346e2c7-e4a6-47bf-bf64-5f6f575160e2
md"""
Table 7.1 provides information about income distribution in Pandora's economy. Take into account that the income aggregates in the table below are net of income taxation imposed by the government (that is, their values are after-income taxation):
"""

# ╔═╡ 10ff11f6-2505-450e-be2c-2be8ffce38e9
md"""
!!! note "Table 7.1: Income distribution in Pandora's economy (2020)"

	| Code     | Income sources             		| Billions of Dollars | 
	| :------  | :------------              		| --------:           | 
	| _ce_     | Compensation of employees      	| 6600      		  | 
	| _ri_     | Rents and  interest 				| 2500      		  |  
	| _rcp_    | Retained corporate profits     	| 1500      		  | 
	| _div_    | Dividends               			|  500      		  | 
	| _nfi_    | Net factor income with the  world 	|  300       		  | 
	| _pit_    | Personal income taxes             	|  2100    			  | 
	| _cit_    | Corporate income taxes             | 700   			  | 
	| _vat_    | VAT taxes         					| 1000      		  | 
	| _dep_    | Depreciation         				| 600     			  | 

"""

# ╔═╡ 6f960cb6-0736-41ed-9e47-5606d2ba698a
ce = 6600; ri = 2500; rcp = 1500; div = 500; nfi = 300; pit = 2100; cit = 700; vat = 1000; dep = 600 ;   

# ╔═╡ 10ba1ff9-47b7-4724-80c4-2c6a9152535d
md"""
**a)** Calculate the level of GDP by the income approach.
"""

# ╔═╡ da2ef284-296c-45c7-9f51-25f0b37cb773
gdp = ce + ri + rcp + div + pit + cit + vat + dep

# ╔═╡ 45bc5044-67d5-43f0-9e9e-2b78055ebfa6
md"""
!!! tip "Answer (a)"
		
	GDP = 15500

"""

# ╔═╡ b18dc5d8-1cbe-4817-bb14-2d8dc4c0e0b1
md"""
**b)** Calculate the level of GNP.
"""

# ╔═╡ c5e578db-2e22-49d8-8ef9-3ad78e9d5bb9
gnp = gdp + nfi

# ╔═╡ 66b48a99-2165-4856-9a55-eed40c7fa9a9
md"""
!!! tip "Answer (b)"
		
	GNP = 15800

"""

# ╔═╡ 21d5595c-8005-49e3-85a1-ebe53625b134
md"""
**c)** Can the information in Table 7.1 be used to calculate the level of GDP by the expenditure approach?
"""

# ╔═╡ 8c129396-cb47-4cd0-91d5-92d59a4012a5
md"""
!!! tip "Answer (c)"
		
	No, because the table above provides no information about the components of GDP by the expenditure approach: $C,I,G,NX$.

"""

# ╔═╡ 991ada8e-9f63-4997-b723-2794060fb049
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 0fb973c6-e2dc-491a-84a5-3dba2480c262
md"""
## Exercise 8. Price indexes and real GDP
"""

# ╔═╡ d884cbdb-32bc-4d97-8a31-78f87765166c
md"""
Consider an economy where only the following goods are produced:

$$\begin{array}{lcccc}
\hline \hline &  \quad \quad \quad \quad   {2004}  &  & \quad \quad \quad \quad {2005} \\
\hline & \text { Quantities } & \text { Prices } & \text { Quantities } & \text { Prices } \\
\hline \text { Computers } & 450 & 6 & 500 & 6 \\
\text { Bikes } & 900 & 4 & 800 & 5 \\
\hline \hline
\end{array}$$
"""

# ╔═╡ ed57ce50-6fed-4977-993d-dd22c8914afc
md"""
We should pass the values of our variables into the notebook:
"""

# ╔═╡ 409acc90-1d55-4720-85fd-07d9960eb985
begin
	# data for 2004     data for 2005
	Qc_04 = 450    ;   	Qc_05 = 500 			# Quantities of Computers
	Qb_04 = 900    ;   	Qb_05 = 800  			# Quantities of Bikes
	Pc_04 =   6    ;  	Pc_05 =   7  			# Price of Computers
	Pb_04 =   4    ; 	Pb_05 =   5  			# Price of Bikes
end;

# ╔═╡ ffab81a6-b645-402b-853b-b2af386b8e81
md"""
👉  **(a)** Why have economists developed the concept of a Price Index?
"""

# ╔═╡ 6ff58637-473f-4cf2-b02c-66d90186d835
md"""
!!! tip "Answer (a)"
		
	Without a Price Index, we can not expurgate the impact of inflation on the real
	value of GDP. If the prices of different goods and services change, and their produced quantities also change, we cannot know the **_real_** value created in one economy.

"""

# ╔═╡ 82469cc2-47e7-4ec6-8f55-c7f3d86f47b2
md"""
---
"""

# ╔═╡ 1755e765-8a3a-4c8b-a19f-e2bd908d2f7c
md"""
👉 **b)** Calculate the value of nominal GDP for 2004 and 2005. Between these two years, how much did nominal GDP grow (as a percentage)?
"""

# ╔═╡ d25a5045-8089-45c7-ac80-108958f1439d
begin
	NGDP04 = Qc_04 * Pc_04 + Qb_04 * Pb_04    		# Quantities 2004 x prices 2004
	NGDP05 = Qc_05 * Pc_05 + Qb_05 * Pb_05    		# Quantities 2005 x prices 2005
	Ngr    = (NGDP05/NGDP04) - 1                	# Growth rate of nominal GDP
	Print([NGDP04 , NGDP05 , Ngr])         			# Print the values of nominal GDP and its growth rate 
end

# ╔═╡ 82faf62f-4e88-456c-ba2b-a8d0bff16e18
md"""
!!! tip "Answer (b)"

	- Nominal GDP 2004 = 6300

	- Nominal GDP 2005 = 7000

	- Growth rate of nominal GDP = 11.11%

"""

# ╔═╡ 54513b73-ed6d-4b4a-9c81-004efa584f01
md"""
---
"""

# ╔═╡ eefa994d-96b5-4ffb-b677-a91395513790
md"""
👉 **c)** Calculate the value of real GDP using **_2004 as the base year_**. Calculate the growth rate of real GDP between 2004 and 2005.
"""

# ╔═╡ df15fe97-3132-4893-93e0-e894c347c37a
begin
	# 2004 as the base year (prices of 2004)
	RGDP04_04 = Qc_04 * Pc_04 + Qb_04 * Pb_04 		# Quantities 2004 x prices 2004
	RGDP05_04 = Qc_05 * Pc_04 + Qb_05 * Pb_04 		# Quantities 2005 x prices 2004
	Rgr_04    = (RGDP05_04/RGDP04_04) - 1 			# Growth rate of real GDP
	Print([RGDP04_04 , RGDP05_04 , Rgr_04]) 		# Print the values of nominal GDP and its growth rate 
end

# ╔═╡ 64aca5c6-f797-4a9d-9995-1fdcc77ef059
md"""
!!! tip "Answer (c)"

	- Real GDP 2004 = 6300

	- Real GDP 2005 = 6200

	- Growth rate of real GDP = --1.587%

"""

# ╔═╡ 372d7305-2f74-4fd0-9953-e852c9540fb9
md"""
---
"""

# ╔═╡ 7194f2f3-6e00-4bc6-8e9d-49e7fcf545eb
md"""
👉 **d)** Calculate the value of real GDP using **_2005 as the base year_**. Calculate the growth rate of real GDP between 2004 and 2005.
"""

# ╔═╡ cc355268-59e0-4165-bbe9-051218168ae8
begin
	# 2005 as the base year (prices of 2005)
	RGDP04_05 = Qc_04 * Pc_05 + Qb_04 * Pb_05
	RGDP05_05 = Qc_05 * Pc_05 + Qb_05 * Pb_05
	Rgr_05    = (RGDP05_05/RGDP04_05) - 1
	Print([RGDP04_05 , RGDP05_05 , Rgr_05])
end

# ╔═╡ 2546322e-d739-4152-a371-9c291ab4bfb4
md"""
!!! tip "Answer (d)"

	- Real GDP 2004 = 7200

	- Real GDP 2005 = 7000

	- Growth rate of real GDP = --2.777%

"""

# ╔═╡ 55398c96-e1c7-4d0c-bcb5-a6314985ab77
md"""
---
"""

# ╔═╡ cfa7d5b3-827b-4f57-b6f5-cc001ff33b13
md"""
👉 **e)** Using 2004 as the base year, calculate the value of the GDP price index for both years. What was the rate of inflation in this economy between 2004 and 2005?
"""

# ╔═╡ 884c2749-df2b-4900-8591-cca66af15082
md"""
!!! hint

	Remember that 

	$$RealGDP = NominalGDP/PriceIndex$$
"""

# ╔═╡ 3cc3fd22-e404-4cbe-9d9f-e90a27072a9a
begin
	IP_04 = NGDP04/RGDP04_04 				# GDP Price Index for 2004
	IP_05 = NGDP05/RGDP05_04 				# GDP Price Index for 2005
	gr_IP = (IP_05-IP_04)/IP_04   			# growth rate of the price index (the rate of inflation)
	Print([IP_04   IP_05   gr_IP])
end

# ╔═╡ b9ed2165-941b-4965-a33b-4adb026eedbb
md"""
!!! tip "Answer (e)"

	- GDP Price Index for 2004 = 1

	- GDP Price Index for 2005 = 1.129

	- The rate of change in the GDP price index gives the inflation rate. Therefore it is equal to 12.9%.

"""

# ╔═╡ fa40cfd7-84a6-44ea-9046-8f34339dc3a0
md"""
---
"""

# ╔═╡ 569fb694-c7c9-40ec-ad99-cb5c2cee063c
md"""
👉 **f)** If we use 2005 as the base year, would we expect to get the same real GDP growth rates and inflation rate as calculated in the case of 2004 as the base year? Why?
"""

# ╔═╡ 40679f74-0237-451e-bf1b-70ca69aa75ea
md"""
!!! tip "Answer (f)"

	- No. We will get different values for the real GDP growth rate and the inflation rate because a change in the base year will change all the real values of macroeconomic aggregates. 

	- It is as if we have to rewrite the history of the economy whenever we change the base year. This is a terrible limitation of fixed-based year indexes, which is why economists developed an index based on "Chain-Weighted prices". 

	- However, we do not have time to cover this type of index. For the curious student, Exercise 12 presents an example.

"""

# ╔═╡ ea76636d-06d8-40ea-b6b5-52be10f0752f
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 53eea00b-84b6-4106-b0a0-6966d02b4230
md"""
## Exercise 9. Unemployment
"""

# ╔═╡ 9d4ef9e8-6a11-4ec8-a3ab-2d1a4168a5a2
md"""
(a) Actual unemployment is the sum of frictional, structural, and cyclical unemployment. Explain the meaning of each type of unemployment.

(b) What is "natural unemployment"?
"""

# ╔═╡ 2e96cd92-bf8b-448d-a5fb-68b3bd3459b2
md"""
	!!! note 
	    Students should try to answer these questions on their own.

	"""

# ╔═╡ 9e64383d-1abb-4034-8b83-2212e4bbf788
md"""
!!! tip "Answer (a)"

	Here

"""

# ╔═╡ 0aabc5fa-725f-4389-ae55-4669ad36178c
md"""
!!! tip "Answer (b)"

	Here

"""

# ╔═╡ 15b231f4-eb48-4ea9-89c4-3afc7872912a
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 90774c46-4424-4489-90f1-73cb1564381b
md"""
## Exercise 10. The natural unemployment rate
"""

# ╔═╡ 2c910e16-e262-4ee8-91eb-9cfc2ab8ba82
md"""
The three figures below present the plots for the US and UK unemployment rates between 1954 and 2016. According to the "Natural Rate of Unemployment" theory, we should expect the **actual** unemployment rate should always be significantly above zero at any period in time due to frictional unemployment and structural unemployment. Do you consider it acceptable, as a theory, that, e.g., the UK had a natural unemployment rate equal to 11.26% in 1984.Q3? Does it look so "natural" to have an unemployment rate higher than 10% of the total labor force? Comment upon this apparent contradiction.
"""

# ╔═╡ 7c06e74a-92fc-449c-b0af-3997e3b66e3f
md"""
!!! tip "Suggested answer"

    It looks acceptable that what is considered as Potential GDP may change with time due to the large that 	may hit the economy; the same for employment and unemployment. 

	However, it looks rather strange for the UK that, in just ten years (the mid-1970s to the mid-1980s), natural unemployment jumped from 4% to more than 11% without any significant shock affecting the UK economy. A bit of common sense will tell us that such an increase does not look very much "natural". Some policy measures must have caused such a dramatic increase in the unemployment rate. However, by definition, policy measures can not affect the level of natural unemployment, so the unemployment level above can not logically be classified as "natural".

"""

# ╔═╡ 2ce95b19-58be-4fd9-b22a-e5a0a449a77e
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 8c6d0a2b-cb11-4284-8ade-39543b006c45
md"""
## Exercise 11. Negative real-interest rates
"""

# ╔═╡ 779d5e72-bfdf-401b-839e-a0a65beb3646
md"""

By definition, the real interest rate (RIR) is the difference between the nominal interest rate and the rate of inflation. It is common to use percentage changes in the Consumer Price Index (CPI) as the proxy for the inflation rate and the Fed Funds Rate (FFR) as the proxy for the nominal interest rate. In this exercise, it is better to use the 3-month maturity Treasury Bills rate (3MTB) instead of the Fed Funds Rate because the Fed has no control over the yields of 3MTB (they are essentially a market phenomenon).

From an economic perspective, it seems highly relevant to see if negative real interest rates are standard behavior or just sporadic facts. From mere intuition, one would be inclined to conclude that such occurrences should be rare because such rates do not make much economic sense. Let us see what has happened in the US since the early 1950s.

In the two figures below, we present the time series of those two variables (3MTB and CPI) and the evolution of the real interest rate (RIR = 3MTB - CPI).

"""

# ╔═╡ 002193a6-32c2-4bc2-9f11-10f8294e4036
md"""
👉 **a)** Why negative real interest rates do not make much sense from an economic point of view?

"""

# ╔═╡ dbcd1e54-050d-4777-8c0c-44012d53ba04
md"""
!!! answer "Answer (a)"

    Negative values for the real interest rate mean that those who lend money to somebody will lose money in **_real terms_** by doing so. Therefore, there are no relevant incentives for banks and other agents to lend money to third parties. Because credit is a fundamental tool to increase efficiency and welfare in our economies, perverse incentives will lead to a bad economic situation.

	But despite negative real interest rates, why do agents still lend money to a third party even with such negative rates? Because not lending money at all will lead to an even worse situation. That is why the presence of negative RIR is so informative about the fact that something is not OK in the economy and the financial markets.

"""

# ╔═╡ 83515d35-19b9-494d-90ba-6708615cc7ee
md"""
👉 **b)** From a financial point of view, negative real interest rates are an aberration. How uncommon is that situation in the US over the last eight decades? 
"""

# ╔═╡ e9ffe69a-08e6-40e6-9b9a-7f2c5844dbeb
md"""
!!! answer "Answer (b)"

	We have 888 months from 1950 until the end of 2023. From those, 337 have negative real interest rate (RIR) values, while 551 have positive values. Therefore, around 38% of the period under consideration was covered by negative RIR. This may raise serious doubts about the popular idea that negative RIR is uncommon and a mere transient feature of modern economies.
"""

# ╔═╡ a0e53085-d09b-44bd-bf8c-7fac9b3a3766
md"""
👉 **c)** In the aftermath of the Covid epidemic, with Western economies displaying a poor performance in economic growth and with significant negative real interest rates (they have been negative since 2003, until May 2023), do you consider that it was risky for governments to borrow to finance investment in infrastructure, health, and education during that period?
"""

# ╔═╡ 797eace8-0095-4df3-964e-6384bb30bc8b
md"""
!!! answer "Suggested answer (c)"

	The exciting part of macroeconomics is that the answer to this question produced in the winter of 2023 has to be slightly different from the same answer in the summer of 2021 (this one is reproduced as part of the following exercise).

	What has changed so much in 2.5 years? The answer is easy: in the summer of 2021, the Western economies were in the bottomless hole of a deflationary process; now, they are emerging from a ravaging inflationary process. The answer to question (b) can not be the same in such disparate situations.

	In the summer of 2021, there were no signs of danger regarding public borrowing to invest in worthy and safe public projects like schools, infrastructure, health, and environmental protection. Inflation expectations were stable and low, long-term interest rates were stable and very low (lower than 1%), and the yields of 30-year public bonds were negative in the most developed countries in the world.

	Now, we are facing two devasting adverse shocks to the world economy: (i) one war of unforeseen consequences (now two wars) and (ii) huge increases in energy prices. These two factors have led inflation to never-seen levels over the last forty years, which led to a sudden jump in short and long-term interest rates. Not surprisingly, the yields of long-term public debt jumped as well. See the following figure for comparing these yields for the AAA Euro area countries for three dates: the 30th of August 2012, 2021, and 2022.

	As one can easily see, macroeconomic instability leads to significant and abrupt changes in the yields of financial assets, public bonds included. Therefore, a word of prudence is required when considering the long-term investment of public funds. 

"""

# ╔═╡ 03d1583c-1f61-470c-99b3-9d5ca071fa66
md"""
The figure below was obtained from here [European Central Bank](https://www.ecb.europa.eu/stats/financial_markets_and_interest_rates/euro_area_yield_curves/html/index.en.html)
"""

# ╔═╡ 21b404da-b630-4b19-8c26-b0f5ad05cf2c
Resource("https://vivaldomendes.org/images/depot/ECB_Yields2.png")

# ╔═╡ d56b77bb-32e6-49ff-8dd8-6559530e8ef4
md"""
👉 **d)** What do you think will happen in the near future? Will real interest rates go back to negative values again? Is it safe for Western governments to borrow in order to finance public investments?
"""

# ╔═╡ 21c5f69b-f04d-4449-a348-ccd3dbf34ebf
md"""
!!! answer "Suggested answer (d)"

	The point is that nobody knows for sure. We know that the economy is very volatile in the short term. Western countries have broken the rampant inflation process we experienced in 2022 and 2023. It seems that inflation is coming to 2% (the target value for most central banks), leading to more stability and lower interest rates. However, the two wars frightening the world and the high instability in energy prices should make future prospects highly uncertain.

	Governments dealing with current high public debt-to-GDP ratios should manage public investments carefully and with prudence.

"""

# ╔═╡ 9effedd9-8288-47fa-97d4-ca8a17866e1b
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 56cd68b7-b242-40d4-8ee6-4836294bfbc3
md"""
## Exercise 12. Answer to (11.b) ... in the summer of 2021
"""

# ╔═╡ 2b04de6c-b157-4c9a-bf48-ec1c8b4c22c2
md"""
!!! answer "Suggested answer (b)"

	In the US, 10-year Treasury Bills rates have moved around 1% over the last couple of years, and long-term inflation expectations are below 2%. This suggests that the scenario of maintaining very low real-interest rates is highly probable. So, as long as public borrowing is allocated to productive activities, which increases production and growth, the risk of public borrowing in the US is negligible.

	In the EuroZone, the situation is even darker. The ECB publishes the yields on long-term bonds with maturities up to 30 years. However surprising, all countries in the EZ with triple AAA public bonds (Germany, France, Netherlands, Austria, Luxembourg, and Belgium) are expected to pay negative interest rates for the entire maturity period on the newly issued public debt (!!): 30 years. See link and figure below.

	Where is the danger with much-needed public infrastructure to repair and much to do about environmental issues? Negative rates for 30 years!
"""

# ╔═╡ 89e96c9a-a582-4ede-90ae-7f0723fa9dcc
md"""
The figure below was obtained from here [European Central Bank](https://www.ecb.europa.eu/stats/financial_markets_and_interest_rates/euro_area_yield_curves/html/index.en.html)
"""

# ╔═╡ 95422345-9f5b-4965-b62c-28c976983ab7
Resource("http://modernmacro.org/resources/Vivaldo/figures/AAA_ratings_2021.png" , :width=>800)

# ╔═╡ 712ac759-1fea-4775-97cb-a3feed435303
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 446aa8e7-13e3-42b3-9fbb-4c0ebeb173da
md"""
## Exercise 13. Chain-Weighted prices (not covered)
"""

# ╔═╡ c3bc75f0-032e-44b3-8b39-0c2f0a413402
md"""
##### This exercise is related to material that was not covered in our classes
"""

# ╔═╡ ff17deea-a50b-493f-89f3-8f4b545595e2
md"""
Consider the information in the following Table:

$$\begin{array}{lcccc}
\hline \hline &  \quad \quad \quad \quad   {2010}  &  & \quad \quad \quad \quad {2011} \\
\hline & \text { Quantities } & \text { Prices } & \text { Quantities } & \text { Prices } \\
\hline \text { Scooters } & 100 & 2 & 120 & 1.5 \\
\text { Cars } & 20 & 50 & 25 & 40 \\
\hline \hline
\end{array}$$
"""

# ╔═╡ 278a7007-e3d0-41f0-a103-45e188f31790
md"""
Let us pass the values of quantities and prices into the notebook:
"""

# ╔═╡ 881d54a3-7cca-4e07-9384-cfe41fcfdb96
qs10 = 100 ; qc10 = 20 ;

# ╔═╡ a85f40b1-9944-4bdc-9cd0-1882ac5f37a8
md"""
while we will use sliders below to assign the values of rest of the entries in that table: 
"""

# ╔═╡ 02a06ef6-98ba-4db4-a2c3-854859a51e8f
begin
	aslider = @bind ps10 Slider(0.0:0.5:4.0, default=2.0, show_value=true)
	bslider = @bind pc10 Slider(20.0:5.0:80.0, default=50.0, show_value=true)
	
	cslider = @bind qs11 Slider(100.0:5.0:140.0, default=120.0, show_value=true)
	dslider = @bind qc11 Slider(10.0:5.0:40.0, default=25.0, show_value=true)
	
	eslider = @bind ps11 Slider(0.0:0.5:3.5, default=1.5, show_value=true)
	fslider = @bind pc11 Slider(20.0:5.0:60.0, default=40.0, show_value=true)
end;


# ╔═╡ 10cabf00-5d76-4674-96f3-7472a89f3b6f
md"""
----
"""

# ╔═╡ 7a520563-ff9b-4a15-b0d3-53715ac2258c
md"""
👉 **a)** Calculate the value of nominal GDP for 2010 and 2011 years, and the growth rate for that period.
"""

# ╔═╡ 3f1f82c0-da35-47bb-a07c-0b6f24c62c4b
begin
	NGDP2010 = qs10 * ps10 + qc10 * pc10
	NGDP2011 = qs11 * ps11 + qc11 * pc11
	Ng   = (NGDP2011/NGDP2010)-1
	Print([NGDP2010  , NGDP2011  , Ng])
end

# ╔═╡ 838e537d-1c1c-4021-addf-82038dfbe5b7
md"""
!!! tip "Answer (a)"
		
	Nominal GDP 2010 = 1200; 

	Nominal GDP 2011 = 1180;

	Growth rate of nominal GDP= -1.66(7)%

"""

# ╔═╡ d1c2f596-dac8-4900-9bd4-8fd6303ebbff
md"""
---
"""

# ╔═╡ b2b0202d-d1b1-4ebd-8dee-58bf4fdb2da0
md"""
👉 **b)** Calculate the level of real GDP for both years, in chained 2010 prices. What is the growth rate of real GDP in this case.
"""

# ╔═╡ 345c9b8e-56ed-4ae1-9a86-d525ed236765
md"""

!!! hint

	Firstly we have to calculate the real GDP growth rate using 2010 as the base year. Then we have to do the same, but for 2011 as the base year. Thirdly, we have to calculate the chained growth rate real GDP as a geometric average of the two previous rates. Finally, we should apply this chained real GDP growth rate to move "forward" or "backward" in calculating real GDP in chain-weighted prices.

"""

# ╔═╡ 6f50b174-5f20-475e-b64e-2783fbca239b
md"""
Step 1:
"""

# ╔═╡ 79ef1593-db6c-48a1-a439-3b6d7343c414
begin
	# 2010 as the base year
	RGDP2010_10 = qs10 * ps10 + qc10 * pc10
	RGDP2011_10 = qs11 * ps10 + qc11 * pc10
	Rg_10   = (RGDP2011_10/RGDP2010_10)-1
	Print([RGDP2010_10 , RGDP2011_10 , Rg_10])
end

# ╔═╡ 7a1451f0-c0e4-4b5d-b1f8-6f3345afdb58
md"""
!!! tip "Answer (b.1)"
		
	Real GDP 2010 (base 2010)= 1200; 

	Real GDP 2011 (base 2010) = 1490;

	Growth rate of nominal GDP (base 2010) = 24.166%

"""

# ╔═╡ 8f94d66c-8115-4b47-b8d5-723e91d56ddb
md"""
------
"""

# ╔═╡ d1b82885-70a7-4986-ba20-229e44ce8813
md"""
Step 2:
"""

# ╔═╡ aaee7709-9154-4093-af98-55228bfee2d4
begin
	# 2011 as the base year
	RGDP2010_11 = qs10 * ps11 + qc10 * pc11
	RGDP2011_11 = qs11 * ps11 + qc11 * pc11
	Rg_11   = (RGDP2011_11/RGDP2010_11)-1
	Print([RGDP2010_11 , RGDP2011_11 , Rg_11])
end

# ╔═╡ 9429853f-5bca-4c37-a770-dfefe58b3555
md"""
!!! tip "Answer (b.2)"
		
	Real GDP 2010 (base 2011)= 1200; 

	Real GDP 2011 (base 2011) = 1490;

	Growth rate of nominal GDP (base 2011) = 24.21%

"""

# ╔═╡ 5406ee07-82d8-4c0b-ab7b-0250e111fdd1
md"""
Step 3:
"""

# ╔═╡ 77390e5f-02a9-4482-af56-774c0c46e781
Rc = sqrt(Rg_10 * Rg_11)

# ╔═╡ 0a7d9e59-6502-4b3d-ba63-56fdf6f525b9
md"""
!!! tip "Answer (b.3)"
		
	The chained growth rate of Real GDP is equal: Rc= 0.2418 

"""

# ╔═╡ eb968506-49e2-40fc-8a8e-824ae95e6988
md"""
Step 4. Multiply the base year real GDP by the chained growth rate to obtain the following year real GDP:
"""

# ╔═╡ 41220b57-fa1e-4405-b7aa-376e659f52c9
begin
	#Real GDP at 2010 chain weighted prices
	RGDP2010_C10 = RGDP2010_10 					# Real GDP 2010 is equal to nominal GDP 2010
	RGDP2011_C10 = RGDP2010_10 * (1+ Rc) 		# Real GDP 2011 is obtained multiplying RGDP2010 by 1+Rc
	Print([RGDP2010_C10 , RGDP2011_C10])
end

# ╔═╡ 6829067e-0e00-4463-853e-15c05635e667
md"""
!!! tip "Answer (b.4)"
		
	Real GDP 2010 (2010 chained prices) = 1200 

	Real GDP 2011 (2010 chained prices) = 1490.263

	Growth rate of real GDP= 24.188%

"""

# ╔═╡ b4aae946-f0a4-4010-ade1-9cf076f47ca4
md"""
---
"""

# ╔═╡ 3b6c4971-7880-499b-a321-9ef3691a172f
md"""

👉 **c)** Calculate the level of real GDP for both years, in chained 2011 prices. What is the growth rate of real GDP in this case.
"""

# ╔═╡ e5e5669a-6c81-4218-b72e-4cfe65c3c1a1
begin
	#Real GDP at 2011 chain weighted prices
	RGDP2010_C11 = RGDP2011_11/(1+ Rc) 			# Real GDP 2010 is obtained dividing RGDP2011 by 1+Rc
	RGDP2011_C11 = RGDP2011_11  				# Real GDP 2011 is equal to nominal GDP 2011
	Print([RGDP2010_C11 , RGDP2011_C11])
end

# ╔═╡ 5688f44a-4697-4087-966a-922aa2516f82
md"""
!!! tip "Answer (c)"
		
	Real GDP 2010 (2011 chained prices) = 950.157 

	Real GDP 2011 (2011 chained prices) = 1180

	Growth rate of real GDP= 24.188%   (the same as in the previous question because they are "chained" growth rates)

"""

# ╔═╡ 085833c8-4e9f-4a00-9815-2115ee1a492b
md"""
----
"""

# ╔═╡ 3650b5bc-5f6b-4b13-9e09-ab8eb560482b
md"""
👉 **(d)** What limitations may be inputed to the construction of Price indexes?
"""

# ╔═╡ 749e8401-ce1b-45e1-88ca-a38fec6500a9
md"""
!!! answer "Answer (d)"

	For the answer, see slide 28 "Some Caution with Price Indexes"

"""

# ╔═╡ 2d7a894f-7331-4052-afc0-e4e28dd580c5
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 47736022-aadf-4e06-9dfd-ef2033139fef
md"""
## Auxiliary cells (do not delete)
"""

# ╔═╡ 9fa87191-7c90-4738-a45a-acd929c8bd1b
  TableOfContents()

# ╔═╡ e58d8a9d-323a-4a02-94cf-30077b8e50b5
md"### CSS code"

# ╔═╡ 755e33a6-dd18-4d64-ab39-497f61bb8132
html"""
<style>
	@media screen {
		main {
			margin: 0 auto;
			max-width: 1550px;
    		padding-left: max(100px, 10%);
    		padding-right: max(380px, 10%); 
            # 383px to accommodate TableOfContents(aside=true)
		}
	}
</style>
"""

# ╔═╡ dbc91676-fb11-40ca-be12-19b2407521ef
begin
      struct TwoColumns{L, R}
    left::L
    right::R
      end
      
      function Base.show(io, mime::MIME"text/html", tc::TwoColumns)
          write(io, """<div style="display: flex;"><div style="flex: 20%;margin-left:19%;">""")
          show(io, mime, tc.left)
          write(io, """</div><div style="flex: 20%;">""")
          show(io, mime, tc.right)
          write(io, """</div></div>""")
      end
end

# ╔═╡ 2f534f42-c4a1-427d-b4cb-563fa0be0cd7
TwoColumns(
md"""

ps10 = $(aslider)

qs11 = $(cslider)

ps11 = $(eslider) 

""",
	
md"""

pc10 = $(bslider)   

qc11 = $(dslider)

pc11 = $(fslider) 

"""
)

# ╔═╡ fe97ab85-2a01-4182-b181-1e4ce3af43de
md"""
### Supporting cells (do not delete)
"""

# ╔═╡ 7d031c88-452e-4c03-bb32-fdb8e9c1d0c7
md"""
##### Data files used in this notebook

- GDP_shares.csv
- Unemp_US_UK.csv
- Real_InterRate.csv

These CSV files must be kept in the same folder as this notebook.
"""

# ╔═╡ bb656856-da28-461b-a03e-451d0c3b0c53
md"""
##### Supporting cells for exercise 10 (The natural unemployment rate)
"""

# ╔═╡ dc933215-3744-4d37-a823-9b2e0f1d499b
begin
	unempl=CSV.read("Unemp_US_UK.csv", DataFrame)
	unempl2 = select(unempl, Not(:1))
end;

# ╔═╡ 2c13ecfe-b545-4596-871c-2844058e473d
begin
	trace3_3 = scatter(;x = unempl.Quarters, y = unempl[:,2], name = "USA", line_color = "Blue", 
						mode = "markers+lines", line_width="0.5", marker_size ="4.5")
	trace3_4 = scatter(;x = unempl.Quarters, y = unempl[:,3], name = "UK", line_color = "Red", 
						mode = "markers+lines", line_width="0.5", marker_size ="4.5")

	layout3_4 = Layout(;height = 450,
				title_text = "Actual unemployment rate in the USA and UK: (1948Q1--2016.Q3)",  
				title_x = 0.5,
				hovermode = "x",
            	xaxis = attr(
                title = "Quarterly obervations",
                #tickformat = "%Y",
                #hoverformat = "%Y-Q%q",
                #tick0 = "1954/01/01",
                dtick = "M60",
        				),
        		xaxis_range = ["1947-07-01", "2017-07-01"],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	fig9_1 = plot([trace3_3 , trace3_4], layout3_4)
end

# ╔═╡ cff06464-55ea-4362-8595-0b2028bddad9
begin
	function hpfilter(unempl2::Array{Float64,1}, lambda::Float64)
    	n = length(unempl2)
    	@assert n >= 4
    	diag2 = lambda*ones(n-2)
    	diag1 = [ -2lambda; -4lambda*ones(n-3); -2lambda ]
    	diag0 = [ 1+lambda; 1+5lambda; (1+6lambda)*ones(n-4); 1+5lambda; 1+lambda ]
    	D = spdiagm(-2 => diag2, -1 => diag1, 0 => diag0, 1 => diag1, 2 => diag2)
    	D\unempl2
	end
end;

# ╔═╡ 015807a9-b434-47be-8274-3cc1027b8565
begin
	hptrend = zeros(276,2)
	for k = 1:2
    	hptrend[:,k] = hpfilter(unempl2[:,k], 1600.0)
	end
	hptrend
end;

# ╔═╡ 2594311f-928a-4508-b401-418d26731de6
begin
	#For the USA
	
	trace3_5 = scatter(;x = unempl.Quarters, y = unempl2[:,1], name = "U",  line_color = "Blue", 
						mode = "markers+lines", line_width="0.5", marker_size ="4.5")
	trace3_6 = scatter(;x = unempl.Quarters, y = hptrend[:,1], name = "Un", line_color = "Red", 
						mode = "markers+lines", line_width="0.5", marker_size ="4.5")

	layout3_6 = Layout(; height = 450,
			title_text = "Actual vs Natural Unemployment (US)", 
				title_x = 0.5,
				hovermode = "x",
            	xaxis = attr(
                title = "Quarterly obervations",
                #tickformat = "%Y",
                #hoverformat = "%Y-Q%q",
                #tick0 = "1954/01/01",
                dtick = "M60",
        				),
        		xaxis_range = ["1947-07-01", "2017-07-01"],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	fig9_2 = plot([trace3_5 , trace3_6], layout3_6)
	
end 

# ╔═╡ b6a95d8d-a765-47f0-b818-0ba42c28e823
begin
	#For the USA
	
	trace3_7 = scatter(;x = unempl.Quarters, y = unempl2[:,2], name = "U",  line_color = "Blue", 
						mode = "markers+lines", line_width="0.5", marker_size ="4.5")
	
	trace3_8 = scatter(;x = unempl.Quarters, y = hptrend[:,2], name = "Un", line_color = "Red", 
						mode = "markers+lines", line_width="0.5", marker_size ="4.5")

	layout3_8 = Layout(height = 450,
				title_text = "Actual vs Natural Unemployment (UK)",  	 			  title_x = 0.5,
				hovermode = "x",
            	xaxis = attr(
                title = "Quarterly obervations",
                #tickformat = "%Y",
                #hoverformat = "%Y-Q%q",
                #tick0 = "1954/01/01",
                dtick = "M60",
        				),
        		xaxis_range = ["1947-07-01", "2017-07-01"],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	fig9_3 = plot([trace3_7 , trace3_8], layout3_8)
	
end 

# ╔═╡ 9fa06f29-5ae3-4d73-b173-9c6c83f6a3d9
md"""
##### Supporting cells for exercise 11 (Negative real interest rates)
"""

# ╔═╡ 81917d80-2fe4-40e6-ad99-61e4838ba781
RIR = CSV.read("Real_InterRate.csv", DataFrame);

# ╔═╡ 53db2a25-d9a9-4c43-b739-b622e07707c7
begin
	fig10_1=plot(RIR.Months, [RIR.CPI RIR[:,2]])
	
	restyle!(fig10_1, name = ["CPI", "3MTB"]) # names CPI and 3MTB on the plotted variables
	
	relayout!(fig10_1, Layout(
		height = 450,
		title_text = "3-Month Treasury Bills rate vs CPI rate: US (1950.M1--2023.M12)", 
		title_x = 0.5, 
		hovermode = "x", 
		yaxis_title = "Percentage points", 
		xaxis_title = "Monthly observations")
	) # introduces a title, y-axis label, and the hover-mode
	
	fig10_1 # Updates the plot
end

# ╔═╡ ce601515-f87c-4641-90bb-d63a6902d27f
begin
	
	trace10_1 = scatter(;x = RIR.Months, y = RIR[:,2]-RIR[:,3], marker=attr(color="darkviolet"),  fill="tozeroy", line_width = 0)

	layout10_1 = Layout(;height = 450,
				title_text = "Real Interest Rate: US (1950.M1--2023.M12)", 
				title_x = 0.5,
				hovermode = "x",
            	xaxis = attr(
                title = "Monthly obervations",
                #tickformat = "%Y",
                #hoverformat = "%Y-Q%q",
                #tick0 = "1960/01/01",
                #dtick = "M120",
        				),
        		xaxis_range = ["1949-01-01", "2024-06-01"],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	fig10_2 = plot([trace10_1], layout10_1)
	
end

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
CSV = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
Dates = "ade2ca70-3891-5945-98fb-dc099432e06a"
PlutoPlotly = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
StatsBase = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"

[compat]
CSV = "~0.10.9"
DataFrames = "~1.7.0"
PlutoPlotly = "~0.6.2"
PlutoUI = "~0.7.49"
StatsBase = "~0.34.3"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.11.3"
manifest_format = "2.0"
project_hash = "e642fa50862d5ce7dc57b48dcac5a8ae2770ede7"

[[deps.AbstractPlutoDingetjes]]
deps = ["Pkg"]
git-tree-sha1 = "6e1d2a35f2f90a4bc7c2ed98079b2ba09c35b83a"
uuid = "6e696c72-6542-2067-7265-42206c756150"
version = "1.3.2"

[[deps.AliasTables]]
deps = ["PtrArrays", "Random"]
git-tree-sha1 = "9876e1e164b144ca45e9e3198d0b689cadfed9ff"
uuid = "66dad0bd-aa9a-41b7-9441-69ab47430ed8"
version = "1.1.3"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.2"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"
version = "1.11.0"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"
version = "1.11.0"

[[deps.CSV]]
deps = ["CodecZlib", "Dates", "FilePathsBase", "InlineStrings", "Mmap", "Parsers", "PooledArrays", "PrecompileTools", "SentinelArrays", "Tables", "Unicode", "WeakRefStrings", "WorkerUtilities"]
git-tree-sha1 = "deddd8725e5e1cc49ee205a1964256043720a6c3"
uuid = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
version = "0.10.15"

[[deps.CodecZlib]]
deps = ["TranscodingStreams", "Zlib_jll"]
git-tree-sha1 = "962834c22b66e32aa10f7611c08c8ca4e20749a9"
uuid = "944b1d66-785c-5afd-91f1-9de20f533193"
version = "0.7.8"

[[deps.ColorSchemes]]
deps = ["ColorTypes", "ColorVectorSpace", "Colors", "FixedPointNumbers", "PrecompileTools", "Random"]
git-tree-sha1 = "26ec26c98ae1453c692efded2b17e15125a5bea1"
uuid = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
version = "3.28.0"

[[deps.ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "b10d0b65641d57b8b4d5e234446582de5047050d"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.5"

[[deps.ColorVectorSpace]]
deps = ["ColorTypes", "FixedPointNumbers", "LinearAlgebra", "Requires", "Statistics", "TensorCore"]
git-tree-sha1 = "a1f44953f2382ebb937d60dafbe2deea4bd23249"
uuid = "c3611d14-8923-5661-9e6a-0046d554d3a4"
version = "0.10.0"

    [deps.ColorVectorSpace.extensions]
    SpecialFunctionsExt = "SpecialFunctions"

    [deps.ColorVectorSpace.weakdeps]
    SpecialFunctions = "276daf66-3868-5448-9aa4-cd146d93841b"

[[deps.Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "362a287c3aa50601b0bc359053d5c2468f0e7ce0"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.11"

[[deps.Compat]]
deps = ["TOML", "UUIDs"]
git-tree-sha1 = "8ae8d32e09f0dcf42a36b90d4e17f5dd2e4c4215"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "4.16.0"
weakdeps = ["Dates", "LinearAlgebra"]

    [deps.Compat.extensions]
    CompatLinearAlgebraExt = "LinearAlgebra"

[[deps.CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "1.1.1+0"

[[deps.Crayons]]
git-tree-sha1 = "249fe38abf76d48563e2f4556bebd215aa317e15"
uuid = "a8cc5b0e-0ffa-5ad4-8c14-923d3ee1735f"
version = "4.1.1"

[[deps.DataAPI]]
git-tree-sha1 = "abe83f3a2f1b857aac70ef8b269080af17764bbe"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.16.0"

[[deps.DataFrames]]
deps = ["Compat", "DataAPI", "DataStructures", "Future", "InlineStrings", "InvertedIndices", "IteratorInterfaceExtensions", "LinearAlgebra", "Markdown", "Missings", "PooledArrays", "PrecompileTools", "PrettyTables", "Printf", "Random", "Reexport", "SentinelArrays", "SortingAlgorithms", "Statistics", "TableTraits", "Tables", "Unicode"]
git-tree-sha1 = "fb61b4812c49343d7ef0b533ba982c46021938a6"
uuid = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
version = "1.7.0"

[[deps.DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "1d0a14036acb104d9e89698bd408f63ab58cdc82"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.20"

[[deps.DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"
version = "1.11.0"

[[deps.DelimitedFiles]]
deps = ["Mmap"]
git-tree-sha1 = "9e2f36d3c96a820c678f2f1f1782582fcf685bae"
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"
version = "1.9.1"

[[deps.DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "2fb1e02f2b635d0845df5d7c167fec4dd739b00d"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.9.3"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.FilePathsBase]]
deps = ["Compat", "Dates"]
git-tree-sha1 = "2ec417fc319faa2d768621085cc1feebbdee686b"
uuid = "48062228-2e41-5def-b9a4-89aafe57970f"
version = "0.9.23"
weakdeps = ["Mmap", "Test"]

    [deps.FilePathsBase.extensions]
    FilePathsBaseMmapExt = "Mmap"
    FilePathsBaseTestExt = "Test"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"
version = "1.11.0"

[[deps.FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "05882d6995ae5c12bb5f36dd2ed3f61c98cbb172"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.5"

[[deps.Future]]
deps = ["Random"]
uuid = "9fa8497b-333b-5362-9e8d-4d0656e87820"
version = "1.11.0"

[[deps.HashArrayMappedTries]]
git-tree-sha1 = "2eaa69a7cab70a52b9687c8bf950a5a93ec895ae"
uuid = "076d061b-32b6-4027-95e0-9a2c6f6d7e74"
version = "0.2.0"

[[deps.Hyperscript]]
deps = ["Test"]
git-tree-sha1 = "179267cfa5e712760cd43dcae385d7ea90cc25a4"
uuid = "47d2ed2b-36de-50cf-bf87-49c2cf4b8b91"
version = "0.0.5"

[[deps.HypertextLiteral]]
deps = ["Tricks"]
git-tree-sha1 = "7134810b1afce04bbc1045ca1985fbe81ce17653"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.5"

[[deps.IOCapture]]
deps = ["Logging", "Random"]
git-tree-sha1 = "b6d6bfdd7ce25b0f9b2f6b3dd56b2673a66c8770"
uuid = "b5f81e59-6552-4d32-b1f0-c071b021bf89"
version = "0.2.5"

[[deps.InlineStrings]]
git-tree-sha1 = "45521d31238e87ee9f9732561bfee12d4eebd52d"
uuid = "842dd82b-1e85-43dc-bf29-5d0ee9dffc48"
version = "1.4.2"

    [deps.InlineStrings.extensions]
    ArrowTypesExt = "ArrowTypes"
    ParsersExt = "Parsers"

    [deps.InlineStrings.weakdeps]
    ArrowTypes = "31f734f8-188a-4ce0-8406-c8a06bd891cd"
    Parsers = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"
version = "1.11.0"

[[deps.InvertedIndices]]
git-tree-sha1 = "6da3c4316095de0f5ee2ebd875df8721e7e0bdbe"
uuid = "41ab1584-1d38-5bbf-9106-f11c6c58b48f"
version = "1.3.1"

[[deps.IrrationalConstants]]
git-tree-sha1 = "e2222959fbc6c19554dc15174c81bf7bf3aa691c"
uuid = "92d709cd-6900-40b7-9082-c6be49f344b6"
version = "0.2.4"

[[deps.IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[deps.JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "31e996f0a15c7b280ba9f76636b3ff9e2ae58c9a"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.4"

[[deps.LaTeXStrings]]
git-tree-sha1 = "dda21b8cbd6a6c40d9d02a73230f9d70fed6918c"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.4.0"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.4"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "8.6.0+0"

[[deps.LibGit2]]
deps = ["Base64", "LibGit2_jll", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"
version = "1.11.0"

[[deps.LibGit2_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll"]
uuid = "e37daf67-58a4-590a-8e99-b0245dd2ffc5"
version = "1.7.2+0"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.11.0+1"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"
version = "1.11.0"

[[deps.LinearAlgebra]]
deps = ["Libdl", "OpenBLAS_jll", "libblastrampoline_jll"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
version = "1.11.0"

[[deps.LogExpFunctions]]
deps = ["DocStringExtensions", "IrrationalConstants", "LinearAlgebra"]
git-tree-sha1 = "13ca9e2586b89836fd20cccf56e57e2b9ae7f38f"
uuid = "2ab3a3ac-af41-5b50-aa03-7779005ae688"
version = "0.3.29"

    [deps.LogExpFunctions.extensions]
    LogExpFunctionsChainRulesCoreExt = "ChainRulesCore"
    LogExpFunctionsChangesOfVariablesExt = "ChangesOfVariables"
    LogExpFunctionsInverseFunctionsExt = "InverseFunctions"

    [deps.LogExpFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    ChangesOfVariables = "9e997f8a-9a97-42d5-a9f1-ce6bfc15e2c0"
    InverseFunctions = "3587e190-3f89-42d0-90ee-14403ec27112"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"
version = "1.11.0"

[[deps.MIMEs]]
git-tree-sha1 = "1833212fd6f580c20d4291da9c1b4e8a655b128e"
uuid = "6c6e2e6c-3030-632d-7369-2d6c69616d65"
version = "1.0.0"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"
version = "1.11.0"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.6+0"

[[deps.Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "ec4f7fbeab05d7747bdf98eb74d130a2a2ed298d"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.2.0"

[[deps.Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"
version = "1.11.0"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2023.12.12"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.OpenBLAS_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Libdl"]
uuid = "4536629a-c528-5b80-bd46-f80d51c5b363"
version = "0.3.27+1"

[[deps.OrderedCollections]]
git-tree-sha1 = "cc4054e898b852042d7b503313f7ad03de99c3dd"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.8.0"

[[deps.Parameters]]
deps = ["OrderedCollections", "UnPack"]
git-tree-sha1 = "34c0e9ad262e5f7fc75b10a9952ca7692cfc5fbe"
uuid = "d96e819e-fc66-5662-9728-84c9c7592b0a"
version = "0.12.3"

[[deps.Parsers]]
deps = ["Dates", "PrecompileTools", "UUIDs"]
git-tree-sha1 = "8489905bcdbcfac64d1daa51ca07c0d8f0283821"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.8.1"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "FileWatching", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "Random", "SHA", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.11.0"
weakdeps = ["REPL"]

    [deps.Pkg.extensions]
    REPLExt = "REPL"

[[deps.PlotlyBase]]
deps = ["ColorSchemes", "Dates", "DelimitedFiles", "DocStringExtensions", "JSON", "LaTeXStrings", "Logging", "Parameters", "Pkg", "REPL", "Requires", "Statistics", "UUIDs"]
git-tree-sha1 = "56baf69781fc5e61607c3e46227ab17f7040ffa2"
uuid = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
version = "0.8.19"

[[deps.PlutoPlotly]]
deps = ["AbstractPlutoDingetjes", "Artifacts", "ColorSchemes", "Colors", "Dates", "Downloads", "HypertextLiteral", "InteractiveUtils", "LaTeXStrings", "Markdown", "Pkg", "PlotlyBase", "PrecompileTools", "Reexport", "ScopedValues", "Scratch", "TOML"]
git-tree-sha1 = "9ebe25fc4703d4112cc418834d5e4c9a4b29087d"
uuid = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
version = "0.6.2"

    [deps.PlutoPlotly.extensions]
    PlotlyKaleidoExt = "PlotlyKaleido"
    UnitfulExt = "Unitful"

    [deps.PlutoPlotly.weakdeps]
    PlotlyKaleido = "f2990250-8cf9-495f-b13a-cce12b45703c"
    Unitful = "1986cc42-f94f-5a68-af5c-568840ba703d"

[[deps.PlutoUI]]
deps = ["AbstractPlutoDingetjes", "Base64", "ColorTypes", "Dates", "FixedPointNumbers", "Hyperscript", "HypertextLiteral", "IOCapture", "InteractiveUtils", "JSON", "Logging", "MIMEs", "Markdown", "Random", "Reexport", "URIs", "UUIDs"]
git-tree-sha1 = "7e71a55b87222942f0f9337be62e26b1f103d3e4"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.61"

[[deps.PooledArrays]]
deps = ["DataAPI", "Future"]
git-tree-sha1 = "36d8b4b899628fb92c2749eb488d884a926614d3"
uuid = "2dfb63ee-cc39-5dd5-95bd-886bf059d720"
version = "1.4.3"

[[deps.PrecompileTools]]
deps = ["Preferences"]
git-tree-sha1 = "5aa36f7049a63a1528fe8f7c3f2113413ffd4e1f"
uuid = "aea7be01-6a6a-4083-8856-8a6e6704d82a"
version = "1.2.1"

[[deps.Preferences]]
deps = ["TOML"]
git-tree-sha1 = "9306f6085165d270f7e3db02af26a400d580f5c6"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.4.3"

[[deps.PrettyTables]]
deps = ["Crayons", "LaTeXStrings", "Markdown", "PrecompileTools", "Printf", "Reexport", "StringManipulation", "Tables"]
git-tree-sha1 = "1101cd475833706e4d0e7b122218257178f48f34"
uuid = "08abe8d2-0d0c-5749-adfa-8a2ac140af0d"
version = "2.4.0"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"
version = "1.11.0"

[[deps.PtrArrays]]
git-tree-sha1 = "1d36ef11a9aaf1e8b74dacc6a731dd1de8fd493d"
uuid = "43287f4e-b6f4-7ad1-bb20-aadabca52c3d"
version = "1.3.0"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "StyledStrings", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"
version = "1.11.0"

[[deps.Random]]
deps = ["SHA"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"
version = "1.11.0"

[[deps.Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[deps.Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "838a3a4188e2ded87a4f9f184b4b0d78a1e91cb7"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.3.0"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.ScopedValues]]
deps = ["HashArrayMappedTries", "Logging"]
git-tree-sha1 = "1147f140b4c8ddab224c94efa9569fc23d63ab44"
uuid = "7e506255-f358-4e82-b7e4-beb19740aa63"
version = "1.3.0"

[[deps.Scratch]]
deps = ["Dates"]
git-tree-sha1 = "3bac05bc7e74a75fd9cba4295cde4045d9fe2386"
uuid = "6c6a2e73-6563-6170-7368-637461726353"
version = "1.2.1"

[[deps.SentinelArrays]]
deps = ["Dates", "Random"]
git-tree-sha1 = "712fb0231ee6f9120e005ccd56297abbc053e7e0"
uuid = "91c51154-3ec4-41a3-a24f-3f23e20d615c"
version = "1.4.8"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"
version = "1.11.0"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"
version = "1.11.0"

[[deps.SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "66e0a8e672a0bdfca2c3f5937efb8538b9ddc085"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.2.1"

[[deps.SparseArrays]]
deps = ["Libdl", "LinearAlgebra", "Random", "Serialization", "SuiteSparse_jll"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
version = "1.11.0"

[[deps.Statistics]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "ae3bb1eb3bba077cd276bc5cfc337cc65c3075c0"
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"
version = "1.11.1"
weakdeps = ["SparseArrays"]

    [deps.Statistics.extensions]
    SparseArraysExt = ["SparseArrays"]

[[deps.StatsAPI]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1ff449ad350c9c4cbc756624d6f8a8c3ef56d3ed"
uuid = "82ae8749-77ed-4fe6-ae5f-f523153014b0"
version = "1.7.0"

[[deps.StatsBase]]
deps = ["AliasTables", "DataAPI", "DataStructures", "LinearAlgebra", "LogExpFunctions", "Missings", "Printf", "Random", "SortingAlgorithms", "SparseArrays", "Statistics", "StatsAPI"]
git-tree-sha1 = "29321314c920c26684834965ec2ce0dacc9cf8e5"
uuid = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"
version = "0.34.4"

[[deps.StringManipulation]]
deps = ["PrecompileTools"]
git-tree-sha1 = "725421ae8e530ec29bcbdddbe91ff8053421d023"
uuid = "892a3eda-7b42-436c-8928-eab12a02cf0e"
version = "0.4.1"

[[deps.StyledStrings]]
uuid = "f489334b-da3d-4c2e-b8f0-e476e12c162b"
version = "1.11.0"

[[deps.SuiteSparse_jll]]
deps = ["Artifacts", "Libdl", "libblastrampoline_jll"]
uuid = "bea87d4a-7f5b-5778-9afe-8cc45184846c"
version = "7.7.0+0"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.3"

[[deps.TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[deps.Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "OrderedCollections", "TableTraits"]
git-tree-sha1 = "598cd7c1f68d1e205689b1c2fe65a9f85846f297"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.12.0"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.0"

[[deps.TensorCore]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1feb45f88d133a655e001435632f019a9a1bcdb6"
uuid = "62fd8b95-f654-4bbd-a8a5-9c27f68ccd50"
version = "0.1.1"

[[deps.Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"
version = "1.11.0"

[[deps.TranscodingStreams]]
git-tree-sha1 = "0c45878dcfdcfa8480052b6ab162cdd138781742"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.11.3"

[[deps.Tricks]]
git-tree-sha1 = "6cae795a5a9313bbb4f60683f7263318fc7d1505"
uuid = "410a4b4d-49e4-4fbc-ab6d-cb71b17b3775"
version = "0.1.10"

[[deps.URIs]]
git-tree-sha1 = "67db6cc7b3821e19ebe75791a9dd19c9b1188f2b"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.5.1"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"
version = "1.11.0"

[[deps.UnPack]]
git-tree-sha1 = "387c1f73762231e86e0c9c5443ce3b4a0a9a0c2b"
uuid = "3a884ed6-31ef-47d7-9d2a-63182c4928ed"
version = "1.0.2"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"
version = "1.11.0"

[[deps.WeakRefStrings]]
deps = ["DataAPI", "InlineStrings", "Parsers"]
git-tree-sha1 = "b1be2855ed9ed8eac54e5caff2afcdb442d52c23"
uuid = "ea10d353-3f73-51f8-a26c-33c1cb351aa5"
version = "1.4.2"

[[deps.WorkerUtilities]]
git-tree-sha1 = "cd1659ba0d57b71a464a29e64dbc67cfe83d54e7"
uuid = "76eceee3-57b5-4d4a-8e66-0e911cebbf60"
version = "1.6.1"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.13+1"

[[deps.libblastrampoline_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850b90-86db-534c-a0d3-1478176c7d93"
version = "5.11.0+0"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.59.0+0"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+2"
"""

# ╔═╡ Cell order:
# ╟─d092fde8-d4cd-40a3-a55d-ab06d322b9fa
# ╟─943edb36-acc8-4b2a-94ab-c544f9eb279b
# ╟─1245aef9-18ce-42f5-af82-c2e6cce70798
# ╟─aa744544-1a69-45b4-aee6-7b9668d7f3cd
# ╟─4618a663-f77e-4ccc-9b00-e84dbd5beb34
# ╠═a145e2fe-a903-11eb-160f-df7ea83fa3e6
# ╟─5930d220-05cc-44ee-946c-55b613b235f8
# ╟─dcd13560-0f73-4aaa-b59a-b7961d8cdc63
# ╟─df9417d2-ebe1-498c-b299-f2b8fdee1084
# ╟─cec1d956-027e-445e-9b9e-db2aacfe71b1
# ╟─0ac99d4a-f5eb-41fd-bed2-4d7ec42ac19c
# ╟─9e05bc6e-a423-41c2-a236-41432276af88
# ╟─72e235dd-e402-483b-9567-28fb05c364f8
# ╟─96e1035c-a937-4385-b5bd-18ee56204651
# ╟─7f063405-715a-498e-8842-ae3f06a2b058
# ╟─adb106a5-23f6-4a1c-b4f0-2f7d26b51709
# ╟─7b68feda-27a4-4ef5-8faf-698be306e3f0
# ╟─a3668c3c-b417-4fc0-8708-5b4bdd937969
# ╟─4e61281c-a3f0-415b-934d-3dacc3750357
# ╟─781d8783-f530-4859-b531-7e1495c53cc4
# ╟─8b753d01-766e-437b-8c45-80a8444564ca
# ╟─1552756d-2d14-47f8-a537-acd59a4031d4
# ╟─52b2eb49-bcec-4901-bb77-39809063af4b
# ╟─d4e6bd4d-b4c0-4817-b53c-4539528d9b00
# ╟─ac61e4b5-0082-47a6-a868-812208e35ee6
# ╟─97a85466-47ff-4869-ba43-a36dea571284
# ╟─839c6baf-dad8-4a97-97ea-d5da508c4263
# ╟─a66f7887-82b2-4826-a8a2-5a95fb070b84
# ╟─d7bce0a6-1a1b-4626-ae34-e546154dcbec
# ╟─a7e5e83a-df8f-4aad-91f9-687a70de5b60
# ╟─d937a483-f279-4ceb-93c9-3d72907a8e76
# ╟─4699b6b5-8c87-4bd0-ac04-5b512b89ba5e
# ╟─4d3108a1-32c0-4400-bc7c-375e0fbcf2ab
# ╟─32cd7b4b-434a-4f35-b732-b6f85a8dc62d
# ╟─78e76c43-6061-48a2-a0d6-c7cc3212e846
# ╟─c089ebb2-9d11-4dfc-814b-f2bf0b08351a
# ╟─af36a976-2a19-4e0b-9dd9-958434b0aef3
# ╟─3f10feb7-8948-4325-9a6d-b69660d22d65
# ╟─dde1db83-f7dc-4d0f-936d-3adf32dc95b7
# ╟─9b633f1a-e96e-414f-81ea-dfc99c1eb465
# ╟─9b01b341-ef94-45c9-a921-a3efd4c0d50a
# ╟─6417e9cc-d4aa-4e21-9188-19f8ea23ad74
# ╟─cbefa2ab-40bb-4c71-a147-00cd779259c5
# ╟─640dd806-873f-4868-b5a8-0db0992e113d
# ╟─5346e2c7-e4a6-47bf-bf64-5f6f575160e2
# ╟─10ff11f6-2505-450e-be2c-2be8ffce38e9
# ╠═6f960cb6-0736-41ed-9e47-5606d2ba698a
# ╟─10ba1ff9-47b7-4724-80c4-2c6a9152535d
# ╠═da2ef284-296c-45c7-9f51-25f0b37cb773
# ╟─45bc5044-67d5-43f0-9e9e-2b78055ebfa6
# ╟─b18dc5d8-1cbe-4817-bb14-2d8dc4c0e0b1
# ╠═c5e578db-2e22-49d8-8ef9-3ad78e9d5bb9
# ╟─66b48a99-2165-4856-9a55-eed40c7fa9a9
# ╟─21d5595c-8005-49e3-85a1-ebe53625b134
# ╟─8c129396-cb47-4cd0-91d5-92d59a4012a5
# ╟─991ada8e-9f63-4997-b723-2794060fb049
# ╟─0fb973c6-e2dc-491a-84a5-3dba2480c262
# ╟─d884cbdb-32bc-4d97-8a31-78f87765166c
# ╟─ed57ce50-6fed-4977-993d-dd22c8914afc
# ╠═409acc90-1d55-4720-85fd-07d9960eb985
# ╟─ffab81a6-b645-402b-853b-b2af386b8e81
# ╟─6ff58637-473f-4cf2-b02c-66d90186d835
# ╟─82469cc2-47e7-4ec6-8f55-c7f3d86f47b2
# ╟─1755e765-8a3a-4c8b-a19f-e2bd908d2f7c
# ╠═d25a5045-8089-45c7-ac80-108958f1439d
# ╟─82faf62f-4e88-456c-ba2b-a8d0bff16e18
# ╟─54513b73-ed6d-4b4a-9c81-004efa584f01
# ╟─eefa994d-96b5-4ffb-b677-a91395513790
# ╠═df15fe97-3132-4893-93e0-e894c347c37a
# ╟─64aca5c6-f797-4a9d-9995-1fdcc77ef059
# ╟─372d7305-2f74-4fd0-9953-e852c9540fb9
# ╟─7194f2f3-6e00-4bc6-8e9d-49e7fcf545eb
# ╠═cc355268-59e0-4165-bbe9-051218168ae8
# ╟─2546322e-d739-4152-a371-9c291ab4bfb4
# ╟─55398c96-e1c7-4d0c-bcb5-a6314985ab77
# ╟─cfa7d5b3-827b-4f57-b6f5-cc001ff33b13
# ╟─884c2749-df2b-4900-8591-cca66af15082
# ╠═3cc3fd22-e404-4cbe-9d9f-e90a27072a9a
# ╟─b9ed2165-941b-4965-a33b-4adb026eedbb
# ╟─fa40cfd7-84a6-44ea-9046-8f34339dc3a0
# ╟─569fb694-c7c9-40ec-ad99-cb5c2cee063c
# ╟─40679f74-0237-451e-bf1b-70ca69aa75ea
# ╟─ea76636d-06d8-40ea-b6b5-52be10f0752f
# ╟─53eea00b-84b6-4106-b0a0-6966d02b4230
# ╟─9d4ef9e8-6a11-4ec8-a3ab-2d1a4168a5a2
# ╟─2e96cd92-bf8b-448d-a5fb-68b3bd3459b2
# ╟─9e64383d-1abb-4034-8b83-2212e4bbf788
# ╟─0aabc5fa-725f-4389-ae55-4669ad36178c
# ╟─15b231f4-eb48-4ea9-89c4-3afc7872912a
# ╟─90774c46-4424-4489-90f1-73cb1564381b
# ╟─2c910e16-e262-4ee8-91eb-9cfc2ab8ba82
# ╟─2c13ecfe-b545-4596-871c-2844058e473d
# ╟─2594311f-928a-4508-b401-418d26731de6
# ╟─b6a95d8d-a765-47f0-b818-0ba42c28e823
# ╟─7c06e74a-92fc-449c-b0af-3997e3b66e3f
# ╟─2ce95b19-58be-4fd9-b22a-e5a0a449a77e
# ╟─8c6d0a2b-cb11-4284-8ade-39543b006c45
# ╟─779d5e72-bfdf-401b-839e-a0a65beb3646
# ╟─53db2a25-d9a9-4c43-b739-b622e07707c7
# ╟─ce601515-f87c-4641-90bb-d63a6902d27f
# ╟─002193a6-32c2-4bc2-9f11-10f8294e4036
# ╟─dbcd1e54-050d-4777-8c0c-44012d53ba04
# ╟─83515d35-19b9-494d-90ba-6708615cc7ee
# ╟─e9ffe69a-08e6-40e6-9b9a-7f2c5844dbeb
# ╟─a0e53085-d09b-44bd-bf8c-7fac9b3a3766
# ╟─797eace8-0095-4df3-964e-6384bb30bc8b
# ╟─03d1583c-1f61-470c-99b3-9d5ca071fa66
# ╟─21b404da-b630-4b19-8c26-b0f5ad05cf2c
# ╟─d56b77bb-32e6-49ff-8dd8-6559530e8ef4
# ╟─21c5f69b-f04d-4449-a348-ccd3dbf34ebf
# ╟─9effedd9-8288-47fa-97d4-ca8a17866e1b
# ╟─56cd68b7-b242-40d4-8ee6-4836294bfbc3
# ╟─2b04de6c-b157-4c9a-bf48-ec1c8b4c22c2
# ╟─89e96c9a-a582-4ede-90ae-7f0723fa9dcc
# ╟─95422345-9f5b-4965-b62c-28c976983ab7
# ╟─712ac759-1fea-4775-97cb-a3feed435303
# ╟─446aa8e7-13e3-42b3-9fbb-4c0ebeb173da
# ╟─c3bc75f0-032e-44b3-8b39-0c2f0a413402
# ╟─ff17deea-a50b-493f-89f3-8f4b545595e2
# ╟─278a7007-e3d0-41f0-a103-45e188f31790
# ╠═881d54a3-7cca-4e07-9384-cfe41fcfdb96
# ╟─a85f40b1-9944-4bdc-9cd0-1882ac5f37a8
# ╟─02a06ef6-98ba-4db4-a2c3-854859a51e8f
# ╟─2f534f42-c4a1-427d-b4cb-563fa0be0cd7
# ╟─10cabf00-5d76-4674-96f3-7472a89f3b6f
# ╟─7a520563-ff9b-4a15-b0d3-53715ac2258c
# ╠═3f1f82c0-da35-47bb-a07c-0b6f24c62c4b
# ╟─838e537d-1c1c-4021-addf-82038dfbe5b7
# ╟─d1c2f596-dac8-4900-9bd4-8fd6303ebbff
# ╟─b2b0202d-d1b1-4ebd-8dee-58bf4fdb2da0
# ╟─345c9b8e-56ed-4ae1-9a86-d525ed236765
# ╟─6f50b174-5f20-475e-b64e-2783fbca239b
# ╠═79ef1593-db6c-48a1-a439-3b6d7343c414
# ╟─7a1451f0-c0e4-4b5d-b1f8-6f3345afdb58
# ╟─8f94d66c-8115-4b47-b8d5-723e91d56ddb
# ╟─d1b82885-70a7-4986-ba20-229e44ce8813
# ╠═aaee7709-9154-4093-af98-55228bfee2d4
# ╟─9429853f-5bca-4c37-a770-dfefe58b3555
# ╟─5406ee07-82d8-4c0b-ab7b-0250e111fdd1
# ╠═77390e5f-02a9-4482-af56-774c0c46e781
# ╟─0a7d9e59-6502-4b3d-ba63-56fdf6f525b9
# ╟─eb968506-49e2-40fc-8a8e-824ae95e6988
# ╠═41220b57-fa1e-4405-b7aa-376e659f52c9
# ╟─6829067e-0e00-4463-853e-15c05635e667
# ╟─b4aae946-f0a4-4010-ade1-9cf076f47ca4
# ╟─3b6c4971-7880-499b-a321-9ef3691a172f
# ╠═e5e5669a-6c81-4218-b72e-4cfe65c3c1a1
# ╟─5688f44a-4697-4087-966a-922aa2516f82
# ╟─085833c8-4e9f-4a00-9815-2115ee1a492b
# ╟─3650b5bc-5f6b-4b13-9e09-ab8eb560482b
# ╟─749e8401-ce1b-45e1-88ca-a38fec6500a9
# ╟─2d7a894f-7331-4052-afc0-e4e28dd580c5
# ╟─47736022-aadf-4e06-9dfd-ef2033139fef
# ╟─9fa87191-7c90-4738-a45a-acd929c8bd1b
# ╟─e58d8a9d-323a-4a02-94cf-30077b8e50b5
# ╟─755e33a6-dd18-4d64-ab39-497f61bb8132
# ╟─dbc91676-fb11-40ca-be12-19b2407521ef
# ╟─fe97ab85-2a01-4182-b181-1e4ce3af43de
# ╟─7d031c88-452e-4c03-bb32-fdb8e9c1d0c7
# ╟─bb656856-da28-461b-a03e-451d0c3b0c53
# ╟─dc933215-3744-4d37-a823-9b2e0f1d499b
# ╟─cff06464-55ea-4362-8595-0b2028bddad9
# ╟─015807a9-b434-47be-8274-3cc1027b8565
# ╟─9fa06f29-5ae3-4d73-b173-9c6c83f6a3d9
# ╟─81917d80-2fe4-40e6-ad99-61e4838ba781
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
